#!/bin/bash
#

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e9|e1[0457]|c[257]>:<[mpich|mvapich2]> <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir()
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`

    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi
    qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
    (( seen_bqual = 0 ))
    for qual in "${qlist[@]}"; do
        case $qual in
            [ec][1-9]*)
                cqual=$qual
                ;;
            mpich|mvapich2)
                (( seen_bqual++ )) && ssi_die "Qualifiers mpich and mvapich2 are mutually exclusive"
                bqual=$qual
                ;;
            *)
                ssi_die "Unrecognized qualifier $qual"
        esac
    done

    if [[ -z "$cqual" ]]; then
        ssi_die "Base qualifier is mandatory"
    fi

    if [[ -z "${extraqual}" ]] && [[ -n "${bqual}" ]]; then
      ssi_die "build type is mandatory."
    fi
    basequal=${cqual}${bqual:+:${bqual}}
 }

set_compiler_flags()
{
    case ${cqual} in
        e[79]|e1[045])
            cxxstandard=14
            ;;
        e1[79])
            cxxstandard=17
            ;;
        c[257])
            cxxstandard=17
            cc=clang
            cxx=clang++
            ;;
        *)
            ssi_die "Unrecognized compiler qualifier $cqual"
    esac
    [[ -n "${extraqual}" ]] && 
    case $extraqual in
      debug)
        relflag=Debug
        ;;
      opt)
        ssi_die "$extraqual build is not supported for this package."
        ;;
      prof)
        relflag=RelWithDebInfo
        ;;
      *)
        ssi_die "Unrecognized extra qualifier $extraqual"
    esac
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
if [[ "$extraqual" == "tar" ]]; then
  unset extraqual
  maketar=tar
fi

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=hep_hpc
origpkgver=v0_11_00
pkgver=${origpkgver//-/_}
ssibuildshims_version=v1_04_14
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtardir=${package}-${pkgdotver}
pkgtarname=${pkgtardir}.tar.bz2

get_this_dir
get_ssibuildshims

source ssi_functions -- # For ssi_die().

set_qualifiers
source define_basics ${define_basics_args:---}
set_compiler_flags

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# Link to ups directory
pkgtopdir=${product_dir}/${package}/${pkgver}
cd "${pkgtopdir}" || ssi_die "Unable to cd into ${pkgtopdir}"
mkdir -p ups
ln -sf "../source/${pkgtardir}/ups/hep_hpc.table" ups/ || ssi_die "Unable to make ups directory"

# Untar source
cd "${sourcedir}" && rm -rf ${pkgtardir} || ssi_die "Unable to cd into ${sourcedir}"
tar xf ${tardir}/${pkgtarname}* || ssi_die "Unable to untar source"

source ${SSIBUILDSHIMS_DIR}/bin/setup_cmake "${product_dir}"|| ssi_die "failed to setup cmake"

cd "${pkgtardir}" || ssi_die "Unable to cd into ${sourcedir}/${pkgtardir}"

# declare now so we can setup
# fake ups declare
[[ -f ups/hep_hpc.table ]] || ssi_die "Unable to find UPS table file in source tarball"

fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || \
  ssi_die "fake setup failed"

ups active

set -x

cd "${blddir}" || ssi_die "Unable to cd into ${blddir}"

# N.B. unusual setting of CMAKE_INSTALL_PREFIX, which is thunked
# internally when WANT_UPS is set.
eval CC=${cc-gcc} CXX=${cxx-g++} \
  cmake -DCMAKE_BUILD_TYPE=${relflag} \
  -DCMAKE_INSTALL_PREFIX=${product_dir} \
  -DCMAKE_CXX_STANDARD=${cxxstandard} \
  -DWANT_UPS=TRUE \
  ${bqual:+-DWANT_MPI=TRUE} \
  ${bqual:+-DWANT_H5PY=FALSE} \
  ${sourcedir}/* || \
  ssi_die "configure failed"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

make -j ${ncore} || ssi_die "make failed"

ctest -j ${ncore} || ssi_die "tests failed"

make install || ssi_die "make install failed"

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# Real ups declare done as part of make install.

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "$package is installed at $(eval echo "\${${package^^*}_FQ_DIR}")"


# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
